#!/usr/bin/env python
# -*- coding: utf-8 -*-

# gui.py
#
# Copyright (c) 2008 Magnun Leno da Silva
#
# Author: Magnun Leno da Silva <magnun.leno@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USAA.

__version__ = "0.9"
__author__ = "Magnun Leno da Silva <magnun.leno@gmail.com>"

import gobject
import gtk
import pygtk
pygtk.require('2.0')
import locale
import gettext
import messages
import os

BASE = None
locale.setlocale(locale.LC_ALL, '')

if os.path.exists('locale') and os.path.exists('data'):
    BASE = "data/icons/"
    gettext.bindtextdomain('usbmanager', './locale')
else:
    BASE = "/usr/share/icons/hicolor/"
    gettext.bindtextdomain('usbmanager', '/usr/share/locale')

gettext.textdomain('usbmanager')
_ = gettext.gettext

SMALL_DEVICE_ICONS = {
               "volume"                 : BASE+"24x24/apps/storage.png",
               "portable_audio_player"  : BASE+"24x24/apps/m_player.png",
              }

BIG_DEVICE_ICONS = {
               "volume"                 : BASE+"64x64/apps/storage.png",
               "portable_audio_player"  : BASE+"64x64/apps/m_player.png",
              }

DEFAULT_BIG_DEVICE_ICON = BASE+"64x64/apps/storage.png"
DEFAULT_SMALL_DEVICE_ICON = BASE+"24x24/apps/storage.png"


class DevicesListView(gtk.TreeView):
    def __init__(self, master):
        gtk.TreeView.__init__(self)
        self.master = master
        self.devices = master.devices

        self.__add_pxbuf_column(_("Label"), 0)
        self.__add_text_column(_("Block Device"), 1)
        self.__add_toggle_column(_("Mounted"), 2)
        self.__add_text_column("UID", 3, False)

        # ListStore
        self.devicesList = gtk.ListStore(str, str, gobject.TYPE_BOOLEAN, str)
        self.set_model(self.devicesList)

        self.connect("row-activated", self.__on_row_double_click)
        self.connect("button-press-event", self.__on_row_click)
        self.connect("button-release-event", self.__on_row_click_leave)

        if len(self.devices) is not 0:
            for udi, device in self.devices:
                self.devicesList.append([device.label, device.block_device, device.is_mounted, udi])

    def __add_pxbuf_column(self, title, id):
        cellpb = gtk.CellRendererPixbuf()
        cell = gtk.CellRendererText()

        column = gtk.TreeViewColumn(title)
        column.set_sort_column_id(0)
        column.pack_start(cellpb, False)
        column.pack_start(cell, False)
        column.set_cell_data_func(cellpb, self.__device_pixbuf)
        column.set_cell_data_func(cell, self.__device_name)
        self.append_column(column)

    def __add_text_column(self, title, id, visible=True):
        column = gtk.TreeViewColumn(title, gtk.CellRendererText(), text=id, markup=0)
        column.set_resizable(True)
        column.set_sort_column_id(id)
        column.set_visible(visible)
        self.append_column(column)

    def __add_toggle_column(self, title, id):
        toggle = gtk.CellRendererToggle()
        toggle.set_property('activatable', True)
        toggle.connect('toggled', self.__on_mount_check)
        column = gtk.TreeViewColumn(title, toggle, active=id)
        self.append_column(column)

    def __device_pixbuf(self, column, cell, model, iter):
        device = self.devices.get_device_by_udi(model.get_value(iter, 3))
        pb = None
        icon_name = None

        if SMALL_DEVICE_ICONS.has_key(device.category):
            icon_name = SMALL_DEVICE_ICONS[device.category]
        else:
            icon_name = DEFAULT_SMALL_DEVICE_ICON

        pb = gtk.gdk.pixbuf_new_from_file(icon_name)

        cell.set_property('pixbuf', pb)
        cell.set_property('xalign', 0.5)

    def __device_name(self, column, cell, model, iter):
        cell.set_property('text', model.get_value(iter, 0).ljust(15))
        cell.set_property('xalign', 0)
        device = self.devices.get_device_by_udi(model.get_value(iter, 3))
        if device.blocked:
            cell.set_property("markup", '<span foreground="red">'+device.label+'</span>')

    def refresh_devices_view(self, device):
        current_devices = {}
        for properties in self.devicesList:
            current_devices[properties[3]] = properties

        # Existing Device
        if current_devices.has_key(device.udi):
            properties = current_devices[device.udi]
            properties[0] = device.label
            properties[1] = device.block_device
            properties[2] = device.is_mounted

    def add_new_device(self, device):
        current_devices = {}
        for properties in self.devicesList:
            current_devices[properties[3]] = properties

        if current_devices.has_key(device.udi):
            return

        self.devicesList.append([device.label, device.block_device, device.is_mounted, device.udi])

    def remove_device(self, device):
        for properties in self.devicesList:
            if properties[3] == device.udi:
                self.devicesList.remove(properties.iter)

    def get_selected_device(self):
        selection = self.get_selection()
        model, selection_iter = selection.get_selected()
        if not selection_iter:
            return None
        # Get label
        return self.devices.get_device_by_udi(model.get_value(selection_iter,3))

    def __on_row_double_click(self, widget, column = None, pointer = None):
        device = self.get_selected_device()
        if device is None:
            return
        self.master.create_properties_window(device)

    def __on_row_click(self, widget, event):
        path = widget.get_path_at_pos(int(event.x), int(event.y))
        selection = widget.get_selection()
        if path is None:
            selection.unselect_all()
            self.master.main_window.refresh_device_toolbar(None)

    def __on_row_click_leave(self, widget, event):
        path = widget.get_path_at_pos(int(event.x), int(event.y))
        selection = widget.get_selection()
        model, selection_iter = selection.get_selected()
        if path:
            device = self.devices.get_device_by_udi(model.get_value(selection_iter,3))
            self.master.main_window.refresh_device_toolbar(device)

    def __on_mount_check(self, cell, path):
        for udi, device in self.devices:
            if udi == self.devicesList[path][3]:
                if self.devicesList[path][2]:
                    self.master.umount_device(device)
                else:
                    self.master.mount_device(device)



class ToolBar(gtk.Toolbar):
    def __init__(self, master):
        self.master = master
        # Toolbar
        gtk.Toolbar.__init__(self)
        self.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        self.set_style(gtk.TOOLBAR_ICONS)
        self.tooltips = gtk.Tooltips()
        self.sensitive_buttons = []

        # Properties
        self.properties = self.__add_button_to_toolbar(gtk.STOCK_PROPERTIES, _("Properties"), _("Show volume detailed properties"), self.__on_properties_click)
        self.sensitive_buttons.append(self.properties)

        # Mount/Umount
        self.mount_umount = self.__add_button_to_toolbar(gtk.STOCK_CONNECT, _("Mount/Umount"), _("Mount or Umount device"), self.__on_mount_umount_click)
        self.sensitive_buttons.append(self.mount_umount)

        # Format
        self.format = self.__add_button_to_toolbar(gtk.STOCK_CLEAR, _("Format"), _("Format devices"), self.__on_format_click)
        self.sensitive_buttons.append(self.format)

        #gtk.MenuToolButton
        #self.toolmenu = self.__add_menu_to_toolbar(gtk.STOCK_PREFERENCES, _("Option"), _("Option fot the selected device"), self.__on_menu_click)

        # Space
        self.__add_separator()

        # Quit
        self.__add_button_to_toolbar(gtk.STOCK_QUIT, _("Quit"), _("Quit USBManager"), self.__destroy)

        self.refresh_toolbar_sensitive(False)


    def __add_button_to_toolbar(self, image, label, tooltip, event_func):
        toolbutton = gtk.ToolButton(image)
        toolbutton.set_label(label)
        toolbutton.connect("clicked", event_func)
        self.tooltips.set_tip(toolbutton,tooltip, tooltip)
        self.insert(toolbutton, -1)
        return toolbutton

    def __add_menu_to_toolbar(self, image, label, tooltip, event_func):
        #TODO: Try to show menu when button clicked
        menubutton = gtk.MenuToolButton(gtk.STOCK_PREFERENCES)
        menubutton.connect("clicked", self.__on_menubutton_click)
        self.menu = gtk.Menu()
        menu_item = gtk.MenuItem('some menu item')
        self.menu.append(menu_item)
        self.menu.show_all()
        self.tooltips.set_tip(menubutton,tooltip, tooltip)
        menubutton.set_menu(self.menu)
        self.insert(menubutton, -1)
        return menubutton

    def __add_separator(self):
        sep = gtk.SeparatorToolItem()
        self.insert(sep, -1)

    def refresh_bar(self, device):
        if device is None:
            self.refresh_toolbar_sensitive(False)
            self.refresh_mount_baritem(None)
            return

        self.refresh_toolbar_sensitive(True)
        self.refresh_mount_baritem(device)

    def refresh_toolbar_sensitive(self, sensitive):
        for button in self.sensitive_buttons:
            button.set_sensitive(sensitive)

    def refresh_mount_baritem(self, device):
        if device is None:
            self.mount_umount.set_stock_id(gtk.STOCK_CONNECT)
            self.mount_umount.set_label(_("Mount/Umount"))
            self.tooltips.set_tip(self.mount_umount,_("Mount/Umount selected device"), _("Mount/Umount selected device"))
        elif device.is_mounted:
            self.mount_umount.set_stock_id(gtk.STOCK_CONNECT)
            self.mount_umount.set_label(_("Umount"))
            self.tooltips.set_tip(self.mount_umount,_("Umount selected device"), _("Umount selected device"))
        else:
            self.mount_umount.set_stock_id(gtk.STOCK_DISCONNECT)
            self.mount_umount.set_label(_("Mount"))
            self.tooltips.set_tip(self.mount_umount,_("Mount selected device"), _("Mount selected device"))

    def __on_properties_click(self, udi):
        device = self.master.main_window.devices_list_view.get_selected_device()
        self.master.create_properties_window(device)
        pass


    def __on_format_click(self, widget, envet=None):
        #messages.FunctionNotImemented()
        device = self.master.main_window.devices_list_view.get_selected_device()
        self.master.create_formating_window(device)

    def __on_mount_umount_click(self, widget, event=None):
        device = self.master.main_window.devices_list_view.get_selected_device()
        if device.is_mounted:
            self.master.umount_device(device)
        else:
            self.master.mount_device(device)

    def __on_menubutton_click(self, widget):
        self.menu.popup(None, None, None, 1, 0)
        pass

    def __destroy(self, widget):
        self.master.destroy_main_window()


class CategoryImage(gtk.Image):
    def __init__(self, device):
        gtk.Image.__init__(self)
        icon_name = None

        if BIG_DEVICE_ICONS.has_key(device.category):
            icon_name = BIG_DEVICE_ICONS[device.category]
        else:
            icon_name = DEFAULT_BIG_DEVICE_ICON

        self.set_from_pixbuf(gtk.gdk.pixbuf_new_from_file(icon_name))

class MarkupLabel(gtk.Label):
    def __init__(self, text):
        gtk.Label.__init__(self)
        self.set_markup(text)

class AlignedDescription(gtk.Label):
    def __init__(self, device):
        gtk.Label.__init__(self, device.product + "\n("+device.vendor_full_name+")")
        self.set_alignment(xalign=0.5, yalign=0.5)
        self.set_property("wrap", True)
        self.set_justify(gtk.JUSTIFY_CENTER)

class LabelEntryBox(gtk.HBox):
    def __init__(self, device):
        gtk.HBox.__init__(self, spacing=0)
        self.label = MarkupLabel("<b> "+_("Label")+" : "+"</b>")
        self.label.set_alignment(xalign=1, yalign=0.5)
        self.entry = gtk.Entry(max=15)
        self.entry.set_text(device.label)

        if not device.is_renamable():
            self.entry.set_sensitive(False)
            messages.DeviceNotRenamable(device)

        self.pack_start(self.label, expand=True, fill=False)
        self.pack_start(self.entry, expand=True, fill=True)

class GenericPage(gtk.VBox):
    def __init__(self, device):
        gtk.VBox.__init__(self)
        self.set_spacing(10)
        self.device = device

    def add_framed_table(self, text, rows_count):
        frame = gtk.Frame()
        label = MarkupLabel("<b>"+text+"</b>")
        frame.set_shadow_type(gtk.SHADOW_NONE)
        frame.set_label_widget(label)
        self.pack_start(frame)

        align = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=1.0, yscale=1.0)
        align.set_padding(padding_top=5, padding_bottom=5, padding_left=0, padding_right=0)
        frame.add(align)

        table = gtk.Table(rows=rows_count, columns=2, homogeneous=True)
        table.set_col_spacings(10)
        align.add(table)
        return table

    def add_label_to_table(self, table, text, index):
        label = gtk.Label(text)
        label.set_alignment(xalign=1, yalign=0.5)
        table.attach(label, 0, 1, index, index+1,
                     xoptions=gtk.EXPAND|gtk.FILL,
                     yoptions=gtk.EXPAND|gtk.FILL,
                     xpadding=0, ypadding=2)

    def add_text_value_to_table(self, table, text, index):
        label = gtk.Label(text)
        label.set_alignment(xalign=0, yalign=0.5)
        table.attach(label, 1, 2, index, index+1,
                     xoptions=gtk.EXPAND|gtk.FILL,
                     yoptions=gtk.EXPAND|gtk.FILL,
                     xpadding=0, ypadding=2)
        return label

    def add_checkbtn_to_table(self, table, value, index):
        check = gtk.CheckButton()
        check.set_active(value)
        check.set_alignment(xalign=0, yalign=0.5)
        table.attach(check, 1, 2, index, index+1,
                     xoptions=gtk.EXPAND|gtk.FILL,
                     yoptions=gtk.EXPAND|gtk.FILL,
                     xpadding=0, ypadding=0)
        return check

class BasicInfoPage(GenericPage):
    def __init__(self, device, labels):
        GenericPage.__init__(self, device)
        self.labels = labels

        ## First Table
        table = self.add_framed_table("<b>"+_("Storage Device")+"</b>", 3)
        # Vendor
        self.add_label_to_table(table, self.labels[0], 0)
        self.add_text_value_to_table(table, self.device.vendor, 0)
        # Size
        self.add_label_to_table(table, self.labels[1], 1)
        self.add_text_value_to_table(table, self.device.size, 1)
        # Model
        self.add_label_to_table(table, self.labels[2], 2)
        self.add_text_value_to_table(table, self.device.model, 2)

        ## Second Table
        table = self.add_framed_table("<b>"+_("Mount Options")+"</b>", 3)
        # Is mounted
        self.add_label_to_table(table, self.labels[3], 0)
        self.is_mounted_checkbtn = self.add_checkbtn_to_table(table, self.device.is_mounted, 0)
        # Read Only
        self.add_label_to_table(table, self.labels[4], 1)
        self.read_only_checkbtn = self.add_checkbtn_to_table(table, self.device.read_only, 1)
        self.read_only_checkbtn.set_sensitive(False)
        # Mount Point
        self.add_label_to_table(table, self.labels[5], 2)
        self.mount_point = self.add_text_value_to_table(table, self.device.mount_point, 2)

class AdvancedInfoPage(GenericPage):
    def __init__(self, device, labels):
        GenericPage.__init__(self, device)
        self.labels = labels

        ## First Table
        table = self.add_framed_table("<b>"+_("General")+"</b>", 2)
        # Category
        self.add_label_to_table(table, self.labels[0], 0)
        self.add_text_value_to_table(table, (' '.join(self.device.category.split('_'))).title(), 0)
        # Device Serial Number
        self.add_label_to_table(table, self.labels[1], 1)
        self.add_text_value_to_table(table, self.device.serial, 1)

        ## Second Table
        table = self.add_framed_table("<b>"+_("File System")+"</b>", 3)
        # File System
        self.add_label_to_table(table, self.labels[2], 0)
        self.add_text_value_to_table(table, self.device.fs_type, 0)
        # FS Version
        self.add_label_to_table(table, self.labels[3], 1)
        self.add_text_value_to_table(table, self.device.fs_version, 1)
        # Block Device
        self.add_label_to_table(table, self.labels[4], 2)
        self.add_text_value_to_table(table, self.device.block_device, 2)

class FormatingGenericInfo(gtk.Frame):
    def __init__(self, device, labels, text):
        self.device = device
        self.labels = labels
        gtk.Frame.__init__(self)
        label = MarkupLabel("<b>"+text+"</b>")
        self.set_shadow_type(gtk.SHADOW_NONE)
        self.set_label_widget(label)

        self.align = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=1.0, yscale=1.0)
        self.align.set_padding(padding_top=5, padding_bottom=5, padding_left=10, padding_right=10)
        self.add(self.align)

        self.table = gtk.Table(rows=2, columns=3, homogeneous=False)
        self.table.set_col_spacings(10)
        self.align.add(self.table)


    def add_label_to_table(self, text, index):
        label = gtk.Label(text)
        label.set_alignment(xalign=0, yalign=0.5)
        self.table.attach(label, 0, 1, index, index+1,
                          xoptions=gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL,
                          xpadding=0, ypadding=2)
        self.add_point_sep(index)

    def add_entry(self, text, index):
        entry = gtk.Entry(15)
        entry.set_text(text)
        self.table.attach(entry, 2, 3, index, index+1,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL,
                          xpadding=0, ypadding=2)
        return entry

    def add_combo_box(self, items, index):
        combo = gtk.combo_box_new_text()
        for item in items:
            combo.append_text(item)

        current_fs = None
        if self.device.fs_type == 'vfat':
            current_fs = self.device.fs_version
        else:
            current_fs = self.device.fs_type
        if self.device.fs_type == 'ntfs-3g':
            current_fs = 'NTFS'

        if items.count(current_fs) is 0:
            print ' * Device current FS is unknown (%(fs)s). Assuming FAT16'%{"fs":current_fs}
            combo.set_active(0)
        else:
            print ' * Assuming device current FS:', current_fs
            combo.set_active(items.index(current_fs))

        self.table.attach(combo, 2, 3, index, index+1,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL,
                          xpadding=0, ypadding=2)
        return combo

    def add_text_value_to_table(self, text, index):
        label = gtk.Label(text)
        label.set_alignment(xalign=0, yalign=0.5)
        self.table.attach(label, 2, 3, index, index+1,
                          xoptions=gtk.EXPAND|gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL,
                          xpadding=0, ypadding=2)
        return label

    def add_point_sep(self, index):
        label = gtk.Label(":")
        label.set_alignment(xalign=0.5, yalign=0.5)
        self.table.attach(label, 1, 2, index, index+1,
                          xoptions=gtk.FILL,
                          yoptions=gtk.EXPAND|gtk.FILL,
                          xpadding=0, ypadding=2)


class FormatingDeviceInfo(FormatingGenericInfo):
    def __init__(self, device, labels):
        FormatingGenericInfo.__init__(self, device, labels, _("Device"))

        # Size
        self.add_label_to_table(self.labels[0], 0)
        self.add_text_value_to_table(device.size, 0)
        # Block Device
        self.add_label_to_table(self.labels[1], 1)
        self.add_text_value_to_table(device.block_device, 1)

class FormatingFormatInfo(FormatingGenericInfo):
    def __init__(self, device, labels, suported_fs):
        FormatingGenericInfo.__init__(self, device, labels, _("Formating"))

        # Label
        self.add_label_to_table(self.labels[2], 0)
        self.label_entry = self.add_entry(device.label, 0)
        # FS Type
        self.add_label_to_table(self.labels[3], 1)
        self.fs_combo = self.add_combo_box(suported_fs, 1)

    def get_selected_fs(self):
        return self.fs_combo.get_active_text()

    def get_new_label(self):
        return self.label_entry.get_text()

class FormatingVFatInfo(gtk.Alignment):
    def __init__(self):
        gtk.Alignment.__init__(self, xalign=0.5, yalign=0.5, xscale=1.0, yscale=1.0)
        self.set_padding(padding_top=0, padding_bottom=5, padding_left=10, padding_right=10)

        self.check_bad_blocks = gtk.CheckButton(label=_("Check for Bad Blocks"))
        self.check_bad_blocks.set_active(False)

        self.vbox = gtk.VBox()
        self.add(self.vbox)
        self.vbox.pack_start(self.check_bad_blocks)

    def get_options(self):
        return [self.check_bad_blocks.get_active()]

class FormatingNTFSInfo(gtk.Alignment):
    def __init__(self):
        gtk.Alignment.__init__(self, xalign=0.5, yalign=0.5, xscale=1.0, yscale=1.0)
        self.set_padding(padding_top=0, padding_bottom=5, padding_left=10, padding_right=10)

        self.quick_format = gtk.CheckButton(label=_("Quick Format"))
        self.quick_format.set_active(True)

        self.compression = gtk.CheckButton(label=_("Enable Compression"))
        self.compression.set_active(False)

        self.vbox = gtk.VBox()
        self.add(self.vbox)
        self.vbox.pack_start(self.quick_format)
        self.vbox.pack_start(self.compression)

    def get_options(self):
        return [self.quick_format.get_active(),
                self.compression.get_active()]
