#!/usr/bin/env python
# -*- coding: utf-8 -*-

# gui.py
#
# Copyright (c) 2008 Magnun Leno da Silva
#
# Author: Magnun Leno da Silva <magnun.leno@gmail.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USAA.

__version__ = "0.9"
__author__ = "Magnun Leno da Silva <magnun.leno@gmail.com>"

import gobject
import gtk
import pygtk
pygtk.require('2.0')
import locale
import gettext
import time
import os
from threading import Thread

WINDOW_ICON = None
locale.setlocale(locale.LC_ALL, '')
if os.path.exists("data") and os.path.exists("locale"):
    WINDOW_ICON = "data/icons/64x64/apps/usbmanager.png"
    gettext.bindtextdomain('usbmanager', './locale')
else:
    WINDOW_ICON = "/usr/share/icons/hicolor/64x64/apps/usbmanager.png"
    gettext.bindtextdomain('usbmanager', '/usr/share/locale')



gettext.textdomain('usbmanager')
_ = gettext.gettext

import widgets

class MainWindow(gtk.Window):
    def __init__(self, master):
        self.master = master
        self.devices = self.master.devices

        gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
        self.connect("destroy", self.__on_destroy)
        self.set_border_width(10)
        self.set_size_request(350,250)
        self.set_resizable(False)
        self.set_title(_('USB Volume Storage Manager'))
        self.set_icon_from_file(WINDOW_ICON)
        self.set_position(gtk.WIN_POS_CENTER)
        self.show()
        self.master.main_window = self

        # Main VBox
        self.vbox = gtk.VBox(spacing=3)
        self.add(self.vbox)

        # Toolbar
        self.toolbar = widgets.ToolBar(self.master)
        self.vbox.pack_start(self.toolbar, expand=False)

        # Devices List View
        self.devices_list_view = widgets.DevicesListView(self.master)
        self.vbox.pack_start(self.devices_list_view)

        self.show_all()

    def add_new_device(self, device):
        self.devices_list_view.add_new_device(device)

    def remove_device(self, device):
        self.devices_list_view.remove_device(device)
        self.toolbar.refresh_bar(None)

    def refresh_devices_view(self, device):
        self.devices_list_view.refresh_devices_view(device)
        self.refresh_device_toolbar(device)

    def refresh_device_toolbar(self, device):
        if device is None:
            self.toolbar.refresh_bar(None)
            return
        selected_device = self.devices_list_view.get_selected_device()
        if selected_device is None:
            self.toolbar.refresh_bar(None)
            return

        if selected_device.udi == device.udi:
            self.toolbar.refresh_bar(device)

    def __on_destroy(self, widget):
        self.master.destroy_main_window()

    def gtk_quit(self):
        gtk.main_quit()

    def run(self):
        gtk.main()


class PropertiesWindow(gtk.Dialog):
    def __init__(self, device, master):
        self.master = master
        self.device = device
        self.udi = device.udi
        gtk.Dialog.__init__(self, _("%(device_label)s properties")%{"device_label":device.label},
                            self.master.main_window,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.set_resizable(False)
        self.set_icon_from_file(WINDOW_ICON)
        self.set_border_width(10)
        self.vbox.set_spacing(10)

        self.apply = self.add_button(gtk.STOCK_APPLY, gtk.RESPONSE_ACCEPT)
        self.close = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_REJECT)
        self.apply.set_sensitive(False)


        self.field_labels = [_("Vendor"), _("Size"), _("Model"),
                             _("Is Mounted"), _("Read Only"),
                             _("Mount Point"), _("Category"),
                             _("Serial Number"), _("Type"),
                             _("Version"), _("Block Device")]

        max_len = max([len(label) for label in self.field_labels]) + 3
        self.field_labels = [label.rjust(max_len) + " :" for label in self.field_labels]

        # Add Category Image
        self.category_image = widgets.CategoryImage(self.device)
        self.vbox.pack_start(self.category_image, expand=False, fill=False)
        # Add device Description
        self.vbox.pack_start(widgets.AlignedDescription(self.device))
        # Add Separator
        self.vbox.pack_start(gtk.HSeparator())
        # Add Label & Entry
        label_entry = widgets.LabelEntryBox(self.device)
        self.device_label_entry = label_entry.entry
        self.vbox.pack_start(label_entry, expand=False, fill=False, padding=0)
        # Add Notebook
        self.__add_notebook()

        self.is_mounted_checkbtn.connect("toggled", self.__on_mount_check_toggled)
        self.device_label_entry.connect("key-release-event", self.__on_label_edit)

        self.show_all()

    def __add_notebook(self):
        self.notebook = gtk.Notebook()

        self.basic_page = widgets.BasicInfoPage(self.device, self.field_labels[:6])
        self.notebook.append_page(self.basic_page, widgets.MarkupLabel("<b>"+_("Basic Informations")+"</b>"))
        self.mount_point = self.basic_page.mount_point
        self.is_mounted_checkbtn = self.basic_page.is_mounted_checkbtn

        self.advanced_page = widgets.AdvancedInfoPage(self.device, self.field_labels[6:])
        self.notebook.append_page(self.advanced_page, widgets.MarkupLabel("<b>"+_("Advanced Informations")+"</b>"))

        self.notebook.set_property('homogeneous', True)
        self.vbox.pack_start(self.notebook)

    def update_fields(self):
        # Is mounted check btn
        self.is_mounted_checkbtn.set_active(self.device.is_mounted)
        # Mount point label
        self.mount_point.set_text(self.device.mount_point)
        # Volume label entry
        self.device_label_entry.set_text(self.device.label)

    def get_new_label(self):
        return self.device_label_entry.get_text()

    def restore_orig_label(self):
        self.device_label_entry.set_text(self.device.label)
        self.apply.set_sensitive(False)

    def __on_mount_check_toggled(self, toggle):
        if self.is_mounted_checkbtn.get_active():
            self.master.mount_device(self.device)
        else:
            self.master.umount_device(self.device)

    def __on_label_edit(self, widget, event):
        if self.device_label_entry.get_text() == self.device.label:
            self.apply.set_sensitive(False)
        else:
            self.apply.set_sensitive(True)


class FormatWindow(gtk.Dialog):
    def __init__(self, device, master, suported_fs):
        self.master = master
        self.device = device
        suported_fs.sort()

        self.new_label = None
        self.fs_type = None
        self.fs_options = None
        self.fs_align = None

        gtk.Dialog.__init__(self, _("Formating %(device_label)s")%{"device_label":device.label},
                            self.master.main_window,
                            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT)
        self.set_resizable(False)
        self.set_icon_from_file(WINDOW_ICON)
        self.set_border_width(10)
        self.vbox.set_spacing(10)

        self.field_labels = [_("Size"), _("Block Device"),
                             _("New Label"), _("Files System")]

        max_len = max([len(label) for label in self.field_labels])
        self.field_labels = [label.ljust(max_len) for label in self.field_labels]

        self.device_info = widgets.FormatingDeviceInfo(device, self.field_labels)
        self.vbox.pack_start(self.device_info)

        self.format_info = widgets.FormatingFormatInfo(device, self.field_labels, suported_fs)
        self.vbox.pack_start(self.format_info)

        self.format_info.fs_combo.connect("changed", self.__on_fs_selected)
        self.__on_fs_selected(None)

        btn = gtk.Button(_("Format"))
        btn.set_image(gtk.image_new_from_stock(gtk.STOCK_CLEAR, gtk.ICON_SIZE_BUTTON))
        self.add_action_widget(btn, gtk.RESPONSE_ACCEPT)
        self.close = self.add_button(gtk.STOCK_CLOSE, gtk.RESPONSE_REJECT)

        self.connect("response", self.__on_close)

        self.show_all()


    def __on_fs_selected(self, widget):
        if self.fs_align:
            self.vbox.remove(self.fs_align)
            self.fs_align.destroy()

        selected_fs = self.format_info.get_selected_fs().upper()
        self.fs_align = None
        if selected_fs in ("FAT16", "FAT32"):
            self.fs_align = widgets.FormatingVFatInfo()
        elif selected_fs == "NTFS":
            self.fs_align = widgets.FormatingNTFSInfo()
        elif selected_fs == "EXT2":
            pass
        elif selected_fs == "EXT3":
            pass
        elif selected_fs == "EXT4":
            pass

        self.vbox.pack_start(self.fs_align)
        self.show_all()

    def __on_close(self, dialog, response_id):
        print 'closing'
        self.new_label = self.format_info.get_new_label()
        self.fs_type = self.format_info.get_selected_fs().upper()
        self.fs_options = self.fs_align.get_options()


class ProgressBarPulseThread(Thread):
    def __init__(self, bar):
        Thread.__init__(self)
        self.bar = bar

    def run(self):
        while self.bar.pulse():
            time.sleep(0.2)

class ProgressBar:
    '''
            Simple class to create a progress bar to show progress
    '''
    def __init__(self, master, device):
        '''Initialize the class'''
        self.progress_bar = None
        self.window = None
        self.master = master
        self.device = device
        self.keep_pulsing = False
        self.thread = ProgressBarPulseThread(self)

    def draw(self):
        '''Draw the widgets'''
        # Simple window
        self.window = gtk.Window(gtk.WINDOW_POPUP)
        self.window.set_resizable(False)
        self.window.set_border_width(10)
        self.window.set_position(gtk.WIN_POS_CENTER)
        self.window.set_transient_for(self.master.main_window)
        self.window.set_destroy_with_parent(True)

        # Create a VBox
        vbox = gtk.VBox(False, 5)
        vbox.set_border_width(10)
        self.window.add(vbox)

        # Create the desired progress bar
        self.progress_bar = gtk.ProgressBar()
        self.progress_bar.set_size_request(500, -1)
        self.progress_bar.pulse()
        vbox.pack_start(self.progress_bar, False, False, 0)

        # An expander to hold the textview
        expander = gtk.Expander("<b>"+_("Details")+"</b>")
        expander.set_use_markup(True)
        expander.set_expanded(True)
        # Text buffer
        self.text_buffer = gtk.TextBuffer()
        # The text view
        self.text_view = gtk.TextView(self.text_buffer);
        self.text_view.modify_base(gtk.STATE_NORMAL, gtk.gdk.color_parse("black"))
        self.text_view.modify_text(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
        self.text_view.set_editable(False)
        self.text_view.set_wrap_mode(gtk.WRAP_CHAR)
        self.text_view.set_justification(gtk.JUSTIFY_LEFT)
        self.text_view.set_left_margin(10)
        self.text_view.set_right_margin(10)
        self.text_view.set_size_request(-1, 150)
        # A scroller to the text view
        sw = gtk.ScrolledWindow()
        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        sw.add(self.text_view)
        expander.add(sw)
        vbox.pack_start(expander)

        # close button
        self.close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
        self.close_button.set_sensitive(False)
        self.close_button.connect("clicked", self.__on_close_click)
        # HButtonBox
        btn_box = gtk.HButtonBox()
        btn_box.set_layout(gtk.BUTTONBOX_END)
        btn_box.pack_start(self.close_button)
        vbox.pack_start(btn_box)

        # Show all widgets
        self.window.show_all()

    def addTimer(self):
        self.timer = gobject.timeout_add(5, self.draw_progress, self.progress_bar)

    def write_text(self, text):
        '''Write the text to the label'''
        self.text_buffer.insert(self.text_buffer.get_end_iter(), text)
        self.text_view.scroll_mark_onscreen(self.text_buffer.get_insert())
        #self.pulse()
        while gtk.events_pending():
            gtk.main_iteration(False)

    def draw_end_progress(self, text, widget=None):
        '''Update the progress from the progressbar'''
        self.progress_bar.set_fraction(1)
        self.progress_bar.set_text(text)

    def enable_close(self, erro=None):
        self.keep_pulsing = False
        self.thread.join()
        if erro is None:
            self.draw_end_progress(_("Formating done"))
            self.write_text("\n\n"+_("Done")+"\n")
            print ' * Formationg done!'
        else:
            self.draw_end_progress(_("Error while formating"))
            self.write_text("\n\n*** "+_("Errors:")+"\n")
            self.write_text(erro)
            print ' * Error while formating!'
            print erro

        self.close_button.set_sensitive(True)

    def __on_close_click(self, widget=None):
        '''Close the progress bar'''
        print "Closing..."
        self.window.destroy()
        self.master.destroy_format_progress_bar(self.device)

    def pulse(self, widget=None):
        self.progress_bar.pulse()
        while gtk.events_pending():
            gtk.main_iteration(False)

        return self.keep_pulsing

    def start_pulsing_thread(self):
        self.keep_pulsing = True
        self.thread.start()
